@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -1031,15 +1031,18 @@ static gint mpg123_data_set(SongData *sd, gint read_info)
 				}
 
 			/* set the comment to contain base url */
-			buf = g_strdup(sd->path);
-			p = buf + 7;
-			while (*p != '\0')
+			if (!sd->comment)
 				{
-				if (*p == ':' || *p == '/') *p = '\0';
-				p++;
+				buf = g_strdup(sd->path);
+				p = buf + 7;
+				while (*p != '\0')
+					{
+					if (*p == ':' || *p == '/') *p = '\0';
+					p++;
+					}
+				sd->comment = g_strdup(buf);
+				g_free(buf);
 				}
-			sd->comment = g_strdup(buf);
-			g_free(buf);
 			}
 		return TRUE;
 		}
