@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -26,12 +26,16 @@
 #include <time.h>
 
 #define OGG123_BINARY "ogg123"
+#define OGG123_BINARY_INFO "ogginfo"
+#define OGG123_BINARY_COMMENT "vorbiscomment"
 
 /* ----------------------------------------------------------
    input / output interface to ogg123
    ----------------------------------------------------------*/
 
 gint ogg123_enabled = FALSE;
+gint ogg123_info_found = FALSE;
+gint ogg123_comment_found = FALSE;
 
 gchar *ogg123_device_options = NULL;
 gchar *ogg123_extra_options = NULL;
@@ -42,6 +46,299 @@ static gint ogg123_control_read_id = -1;
 static gint ogg123_child_pid = -1;
 static int ogg123_pipe[2];
 
+/*
+ *----------------------------------------------------------------------------
+ * title/description utils
+ *----------------------------------------------------------------------------
+ */
+
+static const gchar *info_line_value(gchar *text)
+{
+	gchar *ptr, *ptr2;
+
+	ptr = text;
+	while (*ptr != '=' && *ptr != '\0') ptr++;
+	if (*ptr == '\0') return NULL;
+	ptr++;
+
+	/* remove new line char */
+	ptr2 = ptr;
+	while (*ptr2 != '\0')
+		{
+		if (*ptr2 == '\n' || *ptr2 == '\r') *ptr2 = '\0';
+		ptr2++;
+		}
+
+	return ptr;
+}
+
+static gint info_line_key_test(gchar *text, const gchar *key)
+{
+	if (!text || !key) return FALSE;
+	return (strncmp(text, key, strlen(key)) == 0);
+}
+
+static gchar *info_line_key(gchar *text)
+{
+	gchar *ptr;
+	gchar *ret;
+
+	if (!text) return NULL;
+
+	ptr = text;
+	while (*ptr != '=' && *ptr != '\0') ptr++;
+	if (*ptr == '\0') return NULL;
+
+	*ptr = '\0';
+	ret = g_strdup(text);
+	*ptr = '=';
+
+	return ret;
+}
+
+OGGInfo *ogg123_info_get(const gchar *path)
+{
+	OGGInfo *info = NULL;
+	FILE *f;
+	gchar *command;
+	char buf[128];
+
+	if (!ogg123_info_found) return NULL;
+
+	command = g_strdup_printf("%s \"%s\"", OGG123_BINARY_INFO, path);
+	f = popen(command, "r");
+	if (!f)
+		{
+		printf("Failed to run \"%s\n\"", command);
+		g_free(command);
+		return NULL;
+		}
+	g_free(command);
+
+	info = g_new0(OGGInfo, 1);
+
+	while (fgets(buf, sizeof(buf), f) != NULL)
+		{
+		const gchar *value;
+		value = info_line_value(buf);
+
+		if (info_line_key_test(buf, "filename"))
+			{
+			info->pass_integrity = TRUE;
+			}
+		else if (info_line_key_test(buf, "length"))
+			{
+			info->length = (gint)strtol(value, NULL, 10);
+			}
+		else if (info_line_key_test(buf, "header_integrity") ||
+			 info_line_key_test(buf, "stream_integrity"))
+			{
+			info->pass_integrity = info->pass_integrity & (strncmp(value, "pass", 4) == 0);
+			}
+		else if (info_line_key_test(buf, "bitrate_nominal"))
+			{
+			info->bitrate_nominal = (gint)strtol(value, NULL, 10);
+			}
+		else if (info_line_key_test(buf, "bitrate_average"))
+			{
+			info->bitrate_average = (gint)strtol(value, NULL, 10);
+			}
+		else if (info_line_key_test(buf, "bitrate_upper"))
+			{
+			if (strncmp(value, "none", 4) == 0)
+				info->bitrate_upper = 0;
+			else
+				info->bitrate_upper = (gint)strtol(value, NULL, 10);
+			}
+		else if (info_line_key_test(buf, "bitrate_lower"))
+			{
+			if (strncmp(value, "none", 4) == 0)
+				info->bitrate_lower = 0;
+			else
+				info->bitrate_lower = (gint)strtol(value, NULL, 10);
+			}
+		else if (info_line_key_test(buf, "channels"))
+			{
+			info->channels = (gint)strtol(value, NULL, 10);
+			}
+		}
+
+	pclose(f);
+
+	info->bytes = filesize(path);
+
+	return info;
+}
+
+/*
+ * List is ordered key, value, key, value ...
+ * use path_list_free() to free the returned list
+ */
+GList *ogg123_comment_get(const gchar *path)
+{
+	GList *list = NULL;
+	FILE *f;
+	gchar *command;
+	char buf[128];
+
+	if (!ogg123_comment_found) return NULL;
+
+	command = g_strdup_printf("%s -l \"%s\"", OGG123_BINARY_COMMENT, path);
+	f = popen(command, "r");
+	if (!f)
+		{
+		printf("Failed to run \"%s\n\"", command);
+		g_free(command);
+		return NULL;
+		}
+	g_free(command);
+
+	while (fgets(buf, sizeof(buf), f) != NULL)
+		{
+		gchar *key;
+
+		key = info_line_key(buf);
+
+		if (key)
+			{
+			const gchar *value;
+			value = info_line_value(buf);
+
+			if (value)
+				{
+				list = g_list_prepend(list, key);
+				list = g_list_prepend(list, g_strdup(value));
+				key = NULL;
+				}
+			}
+		g_free(key);
+		}
+
+	pclose(f);
+
+	return g_list_reverse(list);
+}
+
+const gchar *ogg123_comment_value(GList *list, const gchar *key)
+{
+	GList *work;
+
+	if (!key) return NULL;
+
+	work = list;
+	while(work)
+		{
+		const gchar *poss = work->data;
+
+		if (strcmp(poss, key) == 0)
+			{
+			if (work->next)
+				{
+				work = work->next;
+				if (work) return (const gchar *)work->data;
+				}
+			return NULL;
+			}
+
+		work = work->next;
+		if (work) work = work->next;
+		}
+
+	return NULL;
+}
+
+gint ogg123_comment_set(const gchar *path, GList *comments)
+{
+	FILE *f;
+	gchar *command;
+	GList *errlist;
+	GList *work;
+	char buf[128];
+	gchar *tmp;
+	
+	if (!ogg123_comment_found) return TRUE;
+	if (!path) return FALSE;
+
+	command = g_strdup_printf("%s -w \"%s\"", OGG123_BINARY_COMMENT, path);
+
+	work = comments;
+	while (work && work->next)
+		{
+		const gchar *key;
+		const gchar *value;
+
+		key = work->data;
+		value = work->next->data;
+
+		if (key && strlen(key) > 0 && value && strlen(value) > 0)
+			{
+			tmp = g_strdup_printf("%s -t \"%s=%s\"", command, key, value);
+			g_free(command);
+			command = tmp;
+			}
+
+		work = work->next;
+		work = work->next;
+		}
+	tmp = g_strdup_printf("%s 2>&1", command);
+	g_free(command);
+	command = tmp;
+
+	f = popen(command, "r");
+	if (!f)
+		{
+		printf("Failed to run \"%s\"\n", command);
+		g_free(command);
+		return FALSE;
+		}
+	g_free(command);
+
+	errlist = NULL;
+	while (fgets(buf, sizeof(buf), f) != NULL)
+		{
+		if (strlen(buf) > 0)
+			{
+			errlist = g_list_append(errlist, g_strdup(buf));
+			}
+		}
+
+	pclose(f);
+
+	if (errlist)
+		{
+		GenericDialog *gd;
+		GtkWidget *tw;
+		gchar *msg;
+
+		msg = g_strdup_printf(_("Failed to set comments for file:\n%s"), path);
+		gd = generic_dialog_new(_("Comment set failed - GQmpeg"), msg,
+					"GQmpeg", "warning", TRUE, NULL, NULL);
+		g_free(msg);
+
+	        generic_dialog_add(gd, _("Ok"), NULL, TRUE);
+
+		tw = gtk_text_new(NULL, NULL);
+		gtk_widget_set_usize(tw, 400, 200);
+		gtk_text_set_editable(GTK_TEXT(tw), FALSE);
+		gtk_box_pack_start(GTK_BOX(gd->vbox), tw, TRUE, TRUE, 5);
+		gtk_widget_show(tw);
+
+		work = errlist;
+		while (work)
+			{
+			gchar *txt = work->data;
+
+			gtk_text_insert(GTK_TEXT(tw), NULL, NULL, NULL, txt, strlen(txt));
+			work = work->next;
+			}
+
+		gtk_widget_show(gd->dialog);
+		}
+
+	path_list_free(errlist);
+	return (errlist == NULL);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * output device utils
@@ -111,42 +408,6 @@ GList *ogg123_get_device_list(void)
  *-----------------------------------------------------------------------------
  */
 
-static GtkWidget *ogg123_info_window(const gchar *file)
-{
-	GtkWidget *vbox;
-	GtkWidget *vbox1;
-	GtkWidget *frame;
-	GtkWidget *label;
-	gchar *buf;
-
-	vbox = gtk_vbox_new(FALSE, 0);
-        gtk_widget_show(vbox);
-
-	buf = g_strconcat(_("Filename: "), filename_from_path(file), NULL);
-	label = gtk_label_new(buf);
-	g_free(buf);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-
-	frame = gtk_frame_new(_("Song info"));
-	gtk_container_border_width (GTK_CONTAINER (frame), 5);
-	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
-	gtk_widget_show(frame);
-
-	vbox1 = gtk_vbox_new(FALSE, 0);
-	gtk_container_border_width (GTK_CONTAINER (vbox1), 5);
-	gtk_container_add(GTK_CONTAINER(frame), vbox1);
-	gtk_widget_show(vbox1);
-
-	label = gtk_label_new(_("Type: ogg bitstream file"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-
-	return vbox;
-}
-
 /*
  *-----------------------------------------------------------------------------
  * child control
@@ -518,9 +779,39 @@ static gint ogg123_data_set(SongData *sd, gint read_info)
 {
 	if (read_info && !sd->info_loaded)
 		{
+		OGGInfo *info;
+		GList *comments;
+
 		sd->info_loaded = TRUE;
 
-		/* FIXME well, this is currently nothing, nada, no extended info */
+		info = ogg123_info_get(sd->path);
+		if (info)
+			{
+			sd->length = info->length;
+			sd->bit_rate = info->bitrate_nominal;
+			sd->channels = info->channels;
+			sd->bit_depth = 16;	/* these two correct ? */
+			sd->khz_rate = 44;
+			if (!info->pass_integrity) sd->flags |= SONG_FLAG_PLAY_FAIL;
+			g_free(info);
+			}
+
+		comments = ogg123_comment_get(sd->path);
+
+		g_free(sd->title);
+		sd->title = g_strdup(ogg123_comment_value(comments, "TITLE"));
+		g_free(sd->artist);
+		sd->artist = g_strdup(ogg123_comment_value(comments, "ARTIST"));
+		g_free(sd->album);
+		sd->album = g_strdup(ogg123_comment_value(comments, "ALBUM"));
+		g_free(sd->genre);
+		sd->genre = g_strdup(ogg123_comment_value(comments, "GENRE"));
+		g_free(sd->comment);
+		sd->comment = g_strdup(ogg123_comment_value(comments, "COMMENT"));
+		g_free(sd->year);
+		sd->year = g_strdup(ogg123_comment_value(comments, "YEAR"));
+
+		path_list_free(comments);
 		}
 
 	return TRUE;
@@ -640,6 +931,18 @@ void ogg123_init(void)
 		printf(_("ogg123 player module disabled.\n"));
 		}
 
+	ogg123_info_found = file_in_path(OGG123_BINARY_INFO);
+	if (!ogg123_info_found)
+		{
+		printf(_("\"%s\" not found.\n"), OGG123_BINARY_INFO);
+		}
+
+	ogg123_comment_found = file_in_path(OGG123_BINARY_COMMENT);
+	if (!ogg123_comment_found)
+		{
+		printf(_("\"%s\" not found, comment support disabled.\n"), OGG123_BINARY_COMMENT);
+		}
+
 	imd = g_new0(IO_ModuleData, 1);
 
 	imd->title = "ogg123";
