@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -15,6 +15,7 @@
 
 #include "players.h"
 #include "playlist.h"
+#include "songinfo.h"
 #include "ui_fileops.h"
 #include "ui_utildlg.h"
 
@@ -542,7 +543,7 @@ static GtkWidget *radio_info_window(const gchar *file)
 	GtkWidget *vbox1;
 	GtkWidget *frame;
 	GtkWidget *label;
-	gchar *buf, *tmp;
+	gchar *buf;
 	double f;
 
 	vbox = gtk_vbox_new(FALSE, 0);
@@ -564,21 +565,13 @@ static GtkWidget *radio_info_window(const gchar *file)
 	gtk_widget_show(vbox1);
 
 	f = radio_path_to_freq(file);
-	buf = g_strdup_printf(_("Frequency: %.2f Mhz"), f);
-	label = gtk_label_new(buf);
+	buf = g_strdup_printf(_("%.2f Mhz"), f);
+	songinfo_add_label(vbox1, _("Frequency"), buf);
 	g_free(buf);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 
-	tmp = radio_path_to_title(file);
-	buf = g_strdup_printf(_("Title: %s"), tmp ? tmp : "");
-	label = gtk_label_new(buf);
+	buf = radio_path_to_title(file);
+	songinfo_add_label(vbox1, _("Title"), buf ? buf : "");
 	g_free(buf);
-	g_free(tmp);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 
 	return vbox;
 }
