@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -28,6 +28,7 @@
 #include "ui2_main.h"
 #include "ui2_util.h"
 #include "ui_fileops.h"
+#include "ui_help.h"
 #include "ui_tabcomp.h"
 #include "types.h"
 #include "window.h"
@@ -824,6 +825,27 @@ static gchar *determine_skin_path(gchar *file)
  *-----------------------------------------------------------------------------
  */
 
+static GtkWidget *help_window = NULL;
+
+static void help_window_destroy_cb(GtkWidget *window, gpointer data)
+{
+	help_window = NULL;
+}
+
+void help_window_show(const gchar *key)
+{
+	if (help_window)
+		{
+		gdk_window_raise(help_window->window);
+		if (key) help_window_set_key(help_window, key);
+		return;
+		}
+
+	help_window = help_window_new(_("Help - GQmpeg"), "GQmpeg", "help",
+				      GQMPEG_SKINDIR "/README", key);
+	gtk_signal_connect(GTK_OBJECT(help_window), "destroy", help_window_destroy_cb, NULL);
+}
+
 static void show_help(void)
 {
 	printf("GQmpeg %s\n", VERSION);
@@ -837,6 +859,7 @@ static void show_help(void)
 	printf(_("  -h,  --help                  displays this message\n"));
 	printf(_("  -v,  --version               shows version info\n"));
 	printf(_("  --blank                      start with empty playlist\n"));
+	printf(_("  --geometry [+|-]xxx[+|-]yyy  specify window position\n"));
 	printf(_("  --crack                      reserved\n"));
 	printf(_(" available ipc_commands:\n"));
 	printf(_("  -p          --play\n"));
@@ -933,6 +956,10 @@ int main (int argc, char *argv[])
 	GList *list_buf = NULL;
 	GList *playlist_buf = NULL;
 	gint tc = 0;
+	gint geometry_set = FALSE;
+	gint geometry_allow = FALSE;
+	gint geometry_x = 0;
+	gint geometry_y = 0;
 
 	/* setup locale, i18n */
 	gtk_set_locale();
@@ -942,7 +969,7 @@ int main (int argc, char *argv[])
 	init_vars();
 
 	/* setup random seed for shuffle mode */
-	srandom (time (NULL));
+	srand(time(NULL));
 
 	/* check that we can find a valid home directory, if not exit semi-gracefully */
 	cbuf = homedir();
@@ -971,58 +998,95 @@ int main (int argc, char *argv[])
 		while (i < argc)
 			{
 			gchar *command_line_path = g_strconcat(current_path, "/", argv[i], NULL);
-			if (!strcmp(argv[i],"--debug"))
+
+			if (strncmp(argv[i], "-", 1) != 0)
+				{
+				if (isdir(command_line_path) || isdir(argv[i]))
+					{
+					if (!default_current_path)
+						{
+						if (isdir(argv[i]))
+							default_current_path = g_strdup(argv[i]);
+						else
+							default_current_path = g_strdup(command_line_path);
+						}
+					}
+				else
+					{
+					if (isfile(argv[i]) || !isfile(command_line_path))
+						{
+						g_free(command_line_path);
+						command_line_path = g_strdup(argv[i]);
+						}
+					if (is_playlist(command_line_path))
+						{
+						playlist_buf = g_list_prepend(playlist_buf, g_strdup(command_line_path));
+						}
+					else
+						{
+						list_buf = g_list_prepend(list_buf, g_strdup(command_line_path));
+						}
+					}
+				}
+			else if (strcmp(argv[i],"--debug") == 0)
 				{
 				debug_mode++;
 				printf("debugging enabled (level %d)\n", debug_mode);
 				}
-			else if ((isdir(command_line_path) || isdir(argv[i])) && !default_current_path)
-				{
-				if (isdir(argv[i]))
-					default_current_path = g_strdup(argv[i]);
-				else
-					default_current_path = g_strdup(command_line_path);
-				}
-			else if (strncmp(argv[i], "-", 1) != 0)
-				{
-				if (isfile(argv[i]) || !isfile(command_line_path))
-					{
-					g_free(command_line_path);
-					command_line_path = g_strdup(argv[i]);
-					}
-				if (is_playlist(command_line_path))
-					{
-					playlist_buf = g_list_prepend(playlist_buf, g_strdup(command_line_path));
-					}
-				else
-					{
-					list_buf = g_list_prepend(list_buf, g_strdup(command_line_path));
-					}
-				}
-			else if (!strcmp(argv[i],"--blank"))
+			else if (strcmp(argv[i],"--blank") == 0)
 				{
 				blank_startup = TRUE;
 				i = argc;
 				}
+			else if (strcmp(argv[i],"--geometry") == 0)
+				{
+				if (i + 1 < argc)
+					{
+					const gchar *geom = argv[i + 1];
+					const gchar *ptr;
+	
+					geometry_set = FALSE;
+	
+					if (*geom == '+' || *geom == '-')
+						{
+						ptr = geom + 1;
+						while (*ptr != '+' && *ptr != '-' && *ptr != '\n') ptr++;
+						if (*ptr != '\n')
+							{
+							buf = g_strndup(geom, ptr - geom);
+							geometry_x = strtol(buf, NULL, 10);
+							g_free(buf);
+							geometry_y = strtol(ptr, NULL, 10);
+							geometry_set = TRUE;
+							}
+						}
+					i++;
+					}
+				else
+					{
+					printf(_("--geometry missing argument\n"));
+					}
+				if (!geometry_set) printf(_("Invalid geometry format ([+|-]xxx[+|-]yyy)\n"));
+				}
 			else if (check_for_ipc_command(&i, argc, argv))
 				{
 				have_ipc_command = TRUE;
 				}
-			else if (!strcmp(argv[i],"--skinhelp"))
+			else if (strcmp(argv[i],"--skinhelp") == 0)
 				{
 				debug_skin = TRUE;
 				printf("skin debugging enabled\n");
 				}
-			else if (!strcmp(argv[i],"--dock"))
+			else if (strcmp(argv[i],"--dock") == 0)
 				{
 				dock_mode = TRUE;
 				}
-			else if (!strcmp(argv[i],"--dockwm"))
+			else if (strcmp(argv[i],"--dockwm") == 0)
 				{
 				dock_mode = TRUE;
 				startup_dockwm = TRUE;
 				}
-			else if (!strncmp(argv[i],"--skin:", 7))
+			else if (strncmp(argv[i],"--skin:", 7) == 0)
 				{
 				if (startup_skin_file)
 					{
@@ -1033,22 +1097,22 @@ int main (int argc, char *argv[])
 					startup_skin_file = determine_skin_path(argv[i] + 7);
 					}
 				}
-			else if (!strcmp(argv[i],"--status"))
+			else if (strcmp(argv[i],"--status") == 0)
 				{
 				ipc_status();
 				return 0;
 				}
-			else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help"))
+			else if (strcmp(argv[i],"-h") == 0 || strcmp(argv[i],"--help") == 0)
 				{
 				show_help();
 				return 0;
 				}
-			else if (!strcmp(argv[i],"-v") || !strcmp(argv[i],"--version"))
+			else if (strcmp(argv[i],"-v") == 0 || strcmp(argv[i],"--version") == 0)
 				{
 				printf("GQmpeg %s\n", VERSION);
 				return 0;
 				}
-			else if (!strcmp(argv[i],"--crack"))
+			else if (strcmp(argv[i],"--crack") == 0)
 				{
 				tc++;
 				}
@@ -1213,26 +1277,25 @@ int main (int argc, char *argv[])
 	g_free(startup_skin_file);
 	startup_skin_file = NULL;
 
+	if (geometry_set)
+		{
+		/* remember state */
+		geometry_allow = main_window->allow_move;
+		/* turn off positioning */
+		main_window->allow_move = FALSE;
+		}
+
 	main_window_set_title(current_song_get_title());
 
 	if (startup_dockwm) windowmaker_setup_dock();
 
-	if (dock_mode) main_window->allow_move = FALSE;
-
 	gtk_widget_show(main_window->window);
 
-	if (slik_remember_position && !dock_mode)
+	if (geometry_set)
 		{
-		/* ensure that window is at least visible */
-		if (window_pos_x < 0 - main_window->skin->width)
-			window_pos_x = 0;
-		if (window_pos_x > gdk_screen_width())
-			window_pos_x = gdk_screen_width() - main_window->skin->width;
-		if (window_pos_y < 0 - main_window->skin->height)
-			window_pos_y = 0;
-		if (window_pos_y > gdk_screen_height())
-			window_pos_y = gdk_screen_height() - main_window->skin->height;
-		gdk_window_move(main_window->window->window, window_pos_x, window_pos_y);
+		/* restore positioning state */
+		main_window->allow_move = geometry_allow;
+		gdk_window_move(main_window->window->window, geometry_x, geometry_y);
 		}
 
 	dnd_init();
@@ -1261,4 +1324,3 @@ int main (int argc, char *argv[])
 	return 0;
 }
 
-
