@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -16,6 +16,7 @@
 #include "btn_funcs.h"
 #include "playlist-window.h"
 #include "preferences.h"
+#include "ui_help.h"
 #include "ui_menu.h"
 
 
@@ -62,6 +63,11 @@ static GtkWidget *menu_preset(void)
 	return menu;
 }
 
+static void show_help(GtkWidget *widget, gpointer data)
+{
+	help_window_show("documentation");
+}
+
 GtkWidget *menu_main(void)
 {
 	GtkWidget *menu;
@@ -117,6 +123,7 @@ GtkWidget *menu_main(void)
 	menu_item_add(menu, _("Run mixer..."), GTK_SIGNAL_FUNC(btn_mixer_pressed), NULL);
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Skin Editor..."), GTK_SIGNAL_FUNC(btn_edit_skin_pressed), NULL);
+	menu_item_add(menu, _("Help..."), show_help, NULL);
 	menu_item_add(menu, _("About..."), GTK_SIGNAL_FUNC(show_about_info), NULL);
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Iconify"), GTK_SIGNAL_FUNC(btn_iconify_pressed), NULL);
@@ -181,6 +188,8 @@ static void menu_playlist_add_defaults(GtkWidget *menu)
 	menu_item_add_divider(menu);
 	menu_item_add(menu, _("Remove"), GTK_SIGNAL_FUNC(plw_playlist_remove_pressed), NULL);
 	menu_item_add(menu, _("Song Info..."), GTK_SIGNAL_FUNC(plw_playlist_songinfo_pressed), NULL);
+	menu_item_add(menu, _("edit Title"), GTK_SIGNAL_FUNC(plw_playlist_edit_title_pressed), NULL);
+	menu_item_add(menu, _("edit Comment"), GTK_SIGNAL_FUNC(plw_playlist_edit_comment_pressed), NULL);
 	menu_item_add_divider(menu);
 }
 
