@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -22,6 +22,7 @@
 #include "playlist-dlg.h"
 #include "songinfo.h"
 #include "utilops.h"
+#include "ui_clist_edit.h"
 #include "ui_fileops.h"
 #include "ui_tabcomp.h"
 #include "ui2_main.h"
@@ -29,9 +30,14 @@
 #include "icon-rarrow.xpm"
 #include "icon-alert.xpm"
 #include "icon-question.xpm"
+#include "icon-custom.xpm"
 
 #define DIR_DEFAULT_HEIGHT 170
 #define DIR_DEFAULT_WIDTH 300
+
+#define PW_SCROLL_DELAY 60
+#define PW_SCROLL_THRESHHOLD 24
+
 /*
  *-----------------------------------------------------------------------------
  * static vars
@@ -48,6 +54,8 @@ static GtkWidget *fileinfolabel;
 static GtkWidget *s_scrolled;
 static GtkWidget *s_hbox;
 
+static gint pw_scroll_id = -1;
+
 /*
  *-----------------------------------------------------------------------------
  * static funcs
@@ -296,6 +304,8 @@ void playlist_window_update_song_icon_by_flags(gint n, SongFlags flags)
 	static GdkPixmap *warning_mask = NULL;
 	static GdkPixmap *question_pixmap = NULL;
 	static GdkPixmap *question_mask = NULL;
+	static GdkPixmap *custom_pixmap = NULL;
+	static GdkPixmap *custom_mask = NULL;
 
 	GdkPixmap *pixmap = NULL;
 	GdkBitmap *mask = NULL;
@@ -322,6 +332,12 @@ void playlist_window_update_song_icon_by_flags(gint n, SongFlags flags)
 		pixmap = question_pixmap;
 		mask = question_mask;
 		}
+	else if (flags & SONG_FLAG_CUSTOMIZED)
+		{
+		if (!custom_pixmap) create_playlist_icon(&custom_pixmap, &custom_mask, (gchar **)icon_custom_xpm);
+		pixmap = custom_pixmap;
+		mask = custom_mask;
+		}
 
 	if (pixmap)
 		{
@@ -973,6 +989,42 @@ void plw_playlist_edit_pressed(void)
 	playlist_dialog_edit_custom_type(playlist_get_data(row));
 }
 
+static gint plw_edit_title_cb(ClistEditData *ced, const gchar *old, const gchar *new, gpointer data)
+{
+	SongData *sd = data;
+
+	/* sanity check, ipc may have changed this behind our back */
+	if (playlist_get_data(ced->row) != sd) return FALSE;
+
+	playlist_data_customize_title(sd, new);
+
+	return (new && strlen(new) > 0);
+}
+
+void plw_playlist_edit_title_pressed(void)
+{
+	gint row = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(playlist_clist)));
+	clist_edit_by_row(GTK_CLIST(playlist_clist), row, 1, plw_edit_title_cb, playlist_get_data(row));
+}
+
+static gint plw_edit_comment_cb(ClistEditData *ced, const gchar *old, const gchar *new, gpointer data)
+{
+	SongData *sd = data;
+
+	/* sanity check, ipc may have changed this behind our back */
+	if (playlist_get_data(ced->row) != sd) return FALSE;
+
+	playlist_data_customize_comment(sd, new);
+
+	return (new && strlen(new) > 0);
+}
+
+void plw_playlist_edit_comment_pressed(void)
+{
+	gint row = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(playlist_clist)));
+	clist_edit_by_row(GTK_CLIST(playlist_clist), row, 8, plw_edit_comment_cb, playlist_get_data(row));
+}
+
 void plw_playlist_move_pressed(void)
 {
 	GList *list = playlist_selection_get_only_file_list();
@@ -1447,6 +1499,83 @@ void playlist_window_filelist_update(void)
 	playlist_window_filelist_populate();
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * playlist window auto-scroll (private)
+ *-----------------------------------------------------------------------------
+ */
+
+static gint playlist_window_drag_scroll_cb(gpointer data)
+{
+	GdkWindow *window;
+	gint x, y;
+	gint w, h;
+	gint amt = 0;
+	GtkAdjustment *adj;
+
+	if (!playlist_window)
+		{
+		pw_scroll_id = -1;
+		return FALSE;
+		}
+
+	window = GTK_CLIST(playlist_clist)->clist_window;
+	gdk_window_get_pointer(window, &x, &y, NULL);
+	gdk_window_get_size(window, &w, &h);
+
+	/* check bounds, remove if out */
+	if (x < 0 || x >= w || y < 0 || y >= h)
+		{
+		pw_scroll_id = -1;
+		return FALSE;
+		}
+
+	if (h < PW_SCROLL_THRESHHOLD * 2)
+		{
+		if (y < h / 2)
+			{
+			amt = 0 - ((h / 2) - y);
+			}
+		else
+			{
+			amt = y - (h / 2);
+			}
+		}
+	else if (y < PW_SCROLL_THRESHHOLD)
+		{
+		amt = 0 - (PW_SCROLL_THRESHHOLD - y);
+		}
+	else if (y >= h - PW_SCROLL_THRESHHOLD)
+		{
+		amt = y - (h - PW_SCROLL_THRESHHOLD);
+		}
+
+	if (amt != 0)
+		{
+		adj = gtk_clist_get_vadjustment(GTK_CLIST(playlist_clist));
+		gtk_adjustment_set_value(adj, MIN(adj->value + amt, adj->upper - adj->page_size));
+		}
+
+	return TRUE;
+}
+
+static void playlist_window_drag_scroll(gint scroll)
+{
+	if (!scroll)
+		{
+		if (pw_scroll_id != -1)
+			{
+			gtk_timeout_remove(pw_scroll_id);
+			}
+		pw_scroll_id = -1;
+		return;
+		}
+
+	if (pw_scroll_id == -1)
+                {
+		pw_scroll_id = gtk_timeout_add(PW_SCROLL_DELAY, playlist_window_drag_scroll_cb, NULL);
+		}
+}
 
 /*
  *-----------------------------------------------------------------------------
@@ -1581,6 +1710,17 @@ static void playlist_window_playlist_set_dnd_data(GtkWidget *widget,
 	g_free(uri_text);
 }
 
+static void playlist_window_dnd_motion(GtkWidget *widget, GdkDragContext *context,
+				       gint x, gint y, guint time, gpointer data)
+{
+	playlist_window_drag_scroll(TRUE);
+}
+
+static void playlist_window_dnd_leave(GtkWidget *widget, GdkDragContext *context, guint time, gpointer data)
+{
+	playlist_window_drag_scroll(FALSE);
+}
+
 static void recalc_rows(GList *list, gint s, gint t)
 {
 	GList *work = list;
@@ -1716,6 +1856,10 @@ static void playlist_window_dnd_init(void)
 			GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_data_received",
 			playlist_window_playlist_get_dnd_data , NULL);
+	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_motion",
+			playlist_window_dnd_motion, NULL);
+	gtk_signal_connect(GTK_OBJECT(playlist_clist), "drag_leave",
+			playlist_window_dnd_leave, NULL);
 
 	gtk_drag_source_set(playlist_clist, GDK_BUTTON2_MASK,
 			playlist_drag_types, playlist_drag_count, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
@@ -1763,6 +1907,8 @@ void playlist_window_hide(void)
 
 	if (!playlist_window) return;
 
+	playlist_window_drag_scroll(FALSE);
+
 	gdk_window_get_root_origin(playlist_window->window, &window_list_x, &window_list_y);
 	gdk_window_get_size(playlist_window->window, &window_list_w, &window_list_h);
 	window_list_div_dir = s_scrolled->allocation.height;
@@ -1873,7 +2019,9 @@ static void playlist_window_build(void)
 	dirlist_clist=gtk_clist_new_with_titles (1, dir_title);
 	gtk_clist_set_column_auto_resize(GTK_CLIST (dirlist_clist), 0, TRUE);
 	gtk_signal_connect (GTK_OBJECT (dirlist_clist), "button_press_event",(GtkSignalFunc) dirlist_press_cb, NULL);
-/*	gtk_widget_set_usize (dirlist_clist, list_width, DIR_DEFAULT_HEIGHT);*/
+#if 0
+	gtk_widget_set_usize (dirlist_clist, list_width, DIR_DEFAULT_HEIGHT);
+#endif
 	gtk_clist_column_titles_passive (GTK_CLIST (dirlist_clist)); 
 	gtk_container_add (GTK_CONTAINER (s_scrolled), dirlist_clist);
 	gtk_widget_show (dirlist_clist);
@@ -1890,7 +2038,9 @@ static void playlist_window_build(void)
 	gtk_clist_set_column_auto_resize(GTK_CLIST (filelist_clist), 0, TRUE);
 	gtk_clist_set_selection_mode(GTK_CLIST(filelist_clist), GTK_SELECTION_EXTENDED);
 	gtk_signal_connect (GTK_OBJECT (filelist_clist), "button_press_event",(GtkSignalFunc) filelist_press_cb, NULL);
-/*	gtk_widget_set_usize (filelist_clist, list_width, -1);*/
+#if 0
+	gtk_widget_set_usize (filelist_clist, list_width, -1);
+#endif
 	gtk_clist_column_titles_passive (GTK_CLIST (filelist_clist)); 
 	gtk_container_add (GTK_CONTAINER (scrolled), filelist_clist);
 	gtk_widget_show (filelist_clist);
