@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -14,6 +14,7 @@
 #include "playlist.h"
 
 #include "display.h"
+#include "ipc.h"
 #include "players.h"
 #include "playlist-window.h"
 #include "types.h"
@@ -53,6 +54,55 @@ static SongData *real_playlist_insert_item(const gchar *path, gint n, gint read_
 static gint real_playlist_remove_item(gint n);
 static gint real_playlist_move_item(gint s, gint t);
 
+/*
+ *-----------------------------------------------------------------------------
+ * misc utils
+ *-----------------------------------------------------------------------------
+ */
+
+static gint path_list_sort_cb(gconstpointer a, gconstpointer b)
+{
+	return strcmp((gchar *)a, (gchar *)b);
+}
+
+GList *path_list_sort(GList *list)
+{
+	return g_list_sort(list, path_list_sort_cb);
+}
+
+GList *path_list_filter(GList *list, gint is_dir_list)
+{
+	GList *work;
+
+	if (disable_filtering && show_dot_files) return list;
+
+	work = list;
+	while (work)
+		{
+		gchar *name = work->data;
+		const gchar *base;
+
+		base = filename_from_path(name);
+
+		if ((!show_dot_files && file_is_hidden(base)) ||
+		    (!is_dir_list && !disable_filtering && !file_is_in_filter(base)) )
+			{
+			GList *link = work;
+			work = work->next;
+			list = g_list_remove_link(list, link);
+			g_free(name);
+			g_list_free(link);
+			}
+		else
+			{
+			work = work->next;
+			}
+		}
+
+	return list;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  * playlist shuffle functions (private)
@@ -68,12 +118,9 @@ static GList *shuffle_list_find_node(gint p)
 
 static void shuffle_list_add (gint p)
 {
-	gint shuffle_p = (float)random() / RAND_MAX * ( g_list_length(shuffle_list) + 1);
+	gint shuffle_p = (float)rand() / RAND_MAX * ( g_list_length(shuffle_list) + 1);
 	GList *node = g_list_nth(playlist, p);
-	if (shuffle_p == g_list_length(shuffle_list))
-		shuffle_list = g_list_append(shuffle_list, node->data);
-	else
-		shuffle_list = g_list_insert(shuffle_list, node->data, shuffle_p);
+	shuffle_list = g_list_insert(shuffle_list, node->data, shuffle_p);
 }
 
 static void shuffle_list_remove (gpointer data)
@@ -181,7 +228,7 @@ void shuffle_list_create(gint start_with_current)
 		node = g_list_nth(work, current_song_get_number());
 		work = g_list_remove(work, node->data);
 
-		p = (float)random() / RAND_MAX * g_list_length(work);
+		p = (float)rand() / RAND_MAX * g_list_length(work);
 		node = g_list_nth(work, p);
 		if (debug_mode) printf("%d ", g_list_index(playlist, node->data));
 		shuffle_list = g_list_prepend(shuffle_list, node->data);
@@ -194,7 +241,7 @@ void shuffle_list_create(gint start_with_current)
 
 	while (work)
 		{
-		gint p =  (float)random() / RAND_MAX * g_list_length(work);
+		gint p =  (float)rand() / RAND_MAX * g_list_length(work);
 		node = g_list_nth(work, p);
 		if (debug_mode) printf("%d ",g_list_index(playlist, node->data));
 		shuffle_list = g_list_prepend(shuffle_list, node->data);
@@ -213,6 +260,43 @@ void shuffle_list_destroy(void)
 	shuffle_list = NULL;
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * misc playlist format functions
+ *-----------------------------------------------------------------------------
+ */
+
+static gchar *line_title(const gchar *text)
+{
+	const gchar *ptr;
+	gint c = 0;
+
+	if (!text || *text == '\0') return NULL;
+
+	ptr = text;
+	while (*ptr != '\0' && *ptr != '\n' && *ptr != '\r')
+		{
+		ptr++;
+		c++;
+		}
+
+	return g_strndup(text, c);
+}
+
+static gchar *m3u_title(const gchar *text)
+{
+	const gchar *ptr;
+
+	if (!text || strncmp(text, "#EXTINF:", 8) != 0) return NULL;
+
+	ptr = text + 8;
+	while (*ptr != ',' && *ptr != '\0') ptr++;
+	if (*ptr != ',') return NULL;
+	ptr++;
+
+	return line_title(ptr);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * playlist loading functions (public)
@@ -226,10 +310,47 @@ static gint  playlist_load_strict = TRUE;
 static gchar *playlist_load_path = NULL;
 static guint32 playlist_load_timer = 0;
 static GList *playlist_load_list = NULL;
+static gint playlist_load_m3u = FALSE;
+static gchar *playlist_load_next_title = NULL;
+static gchar *playlist_load_next_comment = NULL;
+
+static void playlist_load_set_next_title(const gchar *text)
+{
+	g_free(playlist_load_next_title);
+	playlist_load_next_title = g_strdup(text);
+}
+
+static void playlist_load_set_next_comment(const gchar *text)
+{
+	g_free(playlist_load_next_comment);
+	playlist_load_next_comment = g_strdup(text);
+}
 
 static void playlist_load_line_path(const gchar *path)
 {
 	playlist_load_list = g_list_prepend(playlist_load_list, playlist_add(path, FALSE));
+
+	if (playlist_load_next_title || playlist_load_next_comment)
+		{
+		SongData *sd = playlist_load_list->data;
+
+		/* oh well, if customized, we need original data NOW!
+		 * this will hurt background loading a bit :(
+		 * additional note: this should all be moved to a playlist_data_set_custom?? funcs
+		 */
+		playlist_update_by_data(sd);
+
+		if (playlist_load_next_title)
+			{
+			playlist_data_customize_title(sd, playlist_load_next_title);
+			playlist_load_set_next_title(NULL);
+			}
+		if (playlist_load_next_comment)
+			{
+			playlist_data_customize_comment(sd, playlist_load_next_comment);
+			playlist_load_set_next_comment(NULL);
+			}
+		}
 }
 
 static gint playlist_load_list_iterate(void)
@@ -277,6 +398,23 @@ static gint playlist_load_line(FILE *f)
 						display_set_repeat();
 					}
 				}
+			else if (strncmp(s_buf, "#TITLE:", 7) == 0)
+				{
+				playlist_load_set_next_title(line_title(s_buf + 7));
+				}
+			else if (strncmp(s_buf, "#COMMENT:", 9) == 0)
+				{
+				playlist_load_set_next_comment(line_title(s_buf + 9));
+				}
+			else if (playlist_load_m3u && strncmp(s_buf, "#EXTINF:", 8) == 0)
+				{
+				/* m3u song comment */
+				playlist_load_set_next_title(m3u_title(s_buf));
+				}
+			else if (strncmp(s_buf, "#EXTM3U", 7) == 0)
+				{
+				playlist_load_m3u = TRUE;
+				}
 			return 1;
 			}
 		if (*s_buf=='\n') return 1;
@@ -378,6 +516,8 @@ static void playlist_load_end(void)
 		{
 		fclose (playlist_load_fp);
 		playlist_load_fp = NULL;
+
+		ipc_status_send("playlist:done");
 		}
 	if (playlist_load_idle_id != -1)
 		{
@@ -441,9 +581,13 @@ gint playlist_load_start(const gchar *fn, gint append, gint strict, gint parse_b
 
 	playlist_load_append = append;
 	playlist_load_strict = strict;
-	if (playlist_load_path) g_free(playlist_load_path);
+	g_free(playlist_load_path);
 	playlist_load_path = g_strdup(fn);
 
+	playlist_load_m3u = FALSE;
+	playlist_load_set_next_title(NULL);
+	playlist_load_set_next_comment(NULL);
+
 	playlist_load_fp = fopen (playlist_load_path, "r");
 	if (!playlist_load_fp)
 		{
@@ -454,6 +598,8 @@ gint playlist_load_start(const gchar *fn, gint append, gint strict, gint parse_b
 	else
 		{
 		gint c;
+		gchar *buf;
+
 		if (strict)
 			{
 			/* check first line for valid playlist */
@@ -467,6 +613,10 @@ gint playlist_load_start(const gchar *fn, gint append, gint strict, gint parse_b
 				}
 			}
 
+		buf = g_strdup_printf("playlist:read %s", playlist_load_path);
+		ipc_status_send(buf);
+		g_free(buf);
+
 		/* load playlist */
 		if (!append)
 			{
@@ -504,6 +654,8 @@ gint playlist_load_start(const gchar *fn, gint append, gint strict, gint parse_b
 				fclose(playlist_load_fp);
 				playlist_load_fp = NULL;
 				current_song_set(shuffle_list_get_first(), NULL);
+
+				ipc_status_send("playlist:done");
 				}
 			else
 				{
@@ -609,7 +761,24 @@ gint playlist_save(const gchar *fn)
 
 	for (i=0; i < playlist_count; i++)
 		{
-		fprintf(f,"%s\n",playlist_get_item(i));
+		SongData *sd;
+
+		sd = playlist_get_data(i);
+		if (sd)
+			{
+			if (sd->customized)
+				{
+				if (sd->custom_title && sd->title && strlen(sd->title) > 0)
+					{
+					fprintf(f, "#TITLE:%s\n", sd->title);
+					}
+				if (sd->custom_comment && sd->comment && strlen(sd->comment) > 0)
+					{
+					fprintf(f, "#COMMENT:%s\n", sd->comment);
+					}
+				}
+			fprintf(f, "%s\n", playlist_data_get_path(sd));
+			}
 		}
 
 	fprintf(f,"# end of list #\n");
@@ -635,53 +804,37 @@ gint playlist_save(const gchar *fn)
 
 static void playlist_append_dir_func(const gchar *d, gint recursive)
 {
-	DIR *dp;
-	struct dirent *dir;
-	struct stat ent_sbuf;
+	GList *files;
+	GList *dirs;
+	GList *work;
 
-	if((dp = opendir(d))==NULL)
+	if (!path_list(d, &files, &dirs)) return;
+
+	files = path_list_filter(files, FALSE);
+	files = path_list_sort(files);
+
+	work = files;
+	while (work)
 		{
-		/* dir not found */
-		return;
+		gchar *path = work->data;
+		work = work->next;
+		playlist_add(path, TRUE);
 		}
+	path_list_free(files);
 
-	while ((dir = readdir(dp)) != NULL)
+	if (recursive)
 		{
-		/* skips removed files */
-		if (dir->d_ino > 0)
+		dirs = path_list_filter(dirs, TRUE);
+		dirs = path_list_sort(dirs);
+		work = dirs;
+		while (work)
 			{
-			gchar *name;
-			gchar *path;
-
-			name = dir->d_name; 
-			path = g_strconcat(d, "/", name, NULL);
-
-			if (stat(path,&ent_sbuf) >= 0 && S_ISDIR(ent_sbuf.st_mode))
-				{
-				if (!file_is_hidden(name))
-					{
-					if (recursive && strcmp(name, ".") != 0 && strcmp(name, "..") != 0)
-						{
-						playlist_append_dir_func(path, TRUE);
-						}
-					}
-				}
-			else
-				{
-				if (!file_is_hidden(name) && file_is_in_filter(name))
-					{
-					if (!is_playlist(path))
-						{
-						playlist_add(path, TRUE);
-						}
-
-					}
-				}
-			g_free(path);
+			gchar *path = work->data;
+			work = work->next;
+			playlist_append_dir_func(path, TRUE);
 			}
 		}
-
-        closedir(dp);
+	path_list_free(dirs);
 }
 
 void playlist_append_from_dir(const gchar *path, gint recursive)
@@ -1292,6 +1445,9 @@ void playlist_update_by_data(SongData *sd)
 	playlist_length -= sd->length;
 
 	player_module_songdata_update(sd, read_file_information);
+
+	if (current_song_get_data() == sd) display_set_song_text_info(-1, TRUE);
+
 	playlist_window_clist_update_item(n);
 
 	playlist_length += sd->length;
@@ -1355,6 +1511,66 @@ void playlist_clear(void)
 	playlist_window_clist_clear();
 }
 
+/*
+ *-----------------------------------------------------------------------------
+ * playlist data customization functions (public)
+ *-----------------------------------------------------------------------------
+ */
+
+static void playlist_data_customize_chk(SongData *sd)
+{
+	sd->customized = (sd->custom_title || sd->custom_artist || sd->custom_album ||
+			  sd->custom_comment || sd->custom_length);
+	if (sd->customized)
+		{
+		sd->flags |= SONG_FLAG_CUSTOMIZED;
+		}
+	else
+		{
+		sd->flags &= ~SONG_FLAG_CUSTOMIZED;
+		}
+}
+
+void playlist_data_customize_title(SongData *sd, const gchar *title)
+{
+	gint n;
+
+	if (!sd) return;
+
+	if (title && strlen(title) == 0) title = NULL;
+
+	g_free(sd->title);
+	sd->title = g_strdup(title);
+
+	sd->custom_title = (title != NULL);
+	playlist_data_customize_chk(sd);
+
+	if (current_song_get_data() == sd) display_set_song_text_info(-1, TRUE);
+
+	n = playlist_get_index_by_data(sd);
+	playlist_window_clist_update_item(n);
+}
+
+void playlist_data_customize_comment(SongData *sd, const gchar *comment)
+{
+	gint n;
+
+	if (!sd) return;
+
+	if (comment && strlen(comment) == 0) comment = NULL;
+
+	g_free(sd->comment);
+	sd->comment = g_strdup(comment);
+
+	sd->custom_comment = (comment != NULL);
+	playlist_data_customize_chk(sd);
+
+	if (current_song_get_data() == sd) display_set_song_text_info(-1, TRUE);
+
+	n = playlist_get_index_by_data(sd);
+	playlist_window_clist_update_item(n);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * song flag functions (public)
@@ -1414,7 +1630,7 @@ void playlist_randomize(void)
 		GList *node;
 		gint p;
 
-		p =  (float)random() / RAND_MAX * g_list_length(list);
+		p =  (float)rand() / RAND_MAX * g_list_length(list);
 		node = g_list_nth(list, p);
 		playlist = g_list_prepend(playlist, node->data);
 		list = g_list_remove(list, node->data);
@@ -1424,18 +1640,4 @@ void playlist_randomize(void)
 	display_set_song_number(current_song_get_number());
 }
 
-/*
- *-----------------------------------------------------------------------------
- * playlist functions ()
- *-----------------------------------------------------------------------------
- */
-
-
-
-
-
-
-
-
-
 
