@@ -1,6 +1,6 @@
 /*
  * (SLIK) SimpLIstic sKin functions
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -3585,7 +3585,7 @@ static gint distort_cb(gpointer data)
 
 	if (!ui->skin) return FALSE;
 
-	work = g_list_nth(ui->skin->widget_list, (float)random() / (float)RAND_MAX * (float)g_list_length(ui->skin->widget_list));
+	work = g_list_nth(ui->skin->widget_list, (float)rand() / (float)RAND_MAX * (float)g_list_length(ui->skin->widget_list));
 
 	if (work)
 		{
@@ -3601,8 +3601,8 @@ static gint distort_cb(gpointer data)
 		ox = x;
 		oy = y;
 
-		x += (random() > RAND_MAX / 2 ? 1 : -1);
-		y += (random() > RAND_MAX / 2 ? 1 : -1);
+		x += (rand() > RAND_MAX / 2 ? 1 : -1);
+		y += (rand() > RAND_MAX / 2 ? 1 : -1);
 
 		if (x < 0) x = 0;
 		if (y < 0) y = 0;
