@@ -1,6 +1,6 @@
 /*
  * (SLIK) SimpLIstic sKin functions
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -275,7 +275,7 @@ static void ui_show_cb(GtkWidget *widget, gpointer data)
 	gint x, y;
 
 	if (slik_remember_position &&
-	    ui && ui->skin &&
+	    ui && ui->allow_move && ui->skin &&
 	    ui_state_get(filename_from_path(ui->skin_path), ui->skin_mode_key, &x, &y, NULL, NULL))
 		{
 		/* ensure that window is at least visible */
@@ -589,6 +589,11 @@ void ui_title_set(UIData *ui, const gchar *text)
 	gtk_window_set_title(GTK_WINDOW(ui->window), text);
 }
 
+void ui_moveable_set(UIData *ui, gint moveable)
+{
+	ui->allow_move = moveable;
+}
+
 void ui_skin_set(UIData *ui, SkinData *skin, const gchar *path, const gchar *mode_key)
 {
 	gchar *tmp;
@@ -877,7 +882,7 @@ static void ui_slik_about(void)
 	gtk_box_pack_start(GTK_BOX(vbox), pixmap, FALSE, FALSE, 0);
 	gtk_widget_show(pixmap);
 
-	buf = g_strdup_printf(_("SLIK\nSimpLIstic sKin interface\nversion %s\nCopyright (c) 2001 by John Ellis\nhttp://gqmpeg.sourceforge.net\ngqview@email.com\n\nReleased under the GNU Public License"), SLIK_VERSION);
+	buf = g_strdup_printf(_("SLIK\nSimpLIstic sKin interface\nversion %s\nCopyright (c) 2002 by John Ellis\nhttp://gqmpeg.sourceforge.net\ngqview@users.sourceforge.net\n\nReleased under the GNU General Public License"), SLIK_VERSION);
 	label = gtk_label_new(buf);
 	g_free(buf);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
