@@ -1,6 +1,6 @@
 /*
  * (SLIK) SimpLIstic sKin functions
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -28,19 +28,45 @@
 #define COLUMN_PADDING 3	/* stolen from gtkclist.c */
 #define COLUMN_SPACING 1	/* same */
 
-
-static void clist_edit_make_row_visible(ClistEditData *ced, gint row)
+static void clist_edit_get_geom(ClistEditData *ced, gint *x, gint *y, gint *w, gint *h)
 {
-	if (gtk_clist_row_is_visible(ced->clist, row) == GTK_VISIBILITY_FULL) return;
+	*x = GTK_CLIST(ced->clist)->column[(ced->column)].area.x + GTK_CLIST(ced->clist)->hoffset - COLUMN_SPACING - COLUMN_PADDING;
+	*y = GTK_CLIST(ced->clist)->row_height * (ced->row) + (((ced->row) + 1) * 1) + GTK_CLIST(ced->clist)->voffset;
 
-	if (row == 0 || gtk_clist_row_is_visible(ced->clist, row - 1) == GTK_VISIBILITY_NONE)
+	*w = GTK_CLIST(ced->clist)->column[(ced->column)].area.width + (COLUMN_SPACING + COLUMN_PADDING) * 2;
+	*h = GTK_CLIST(ced->clist)->row_height;
+}
+
+static void clist_edit_make_row_visible(ClistEditData *ced)
+{
+	gint nrow, ncol;
+	gfloat frow, fcol;
+	gint x, y, w, h;
+
+	nrow = ncol = -1;
+	frow = fcol = 0.0;
+
+	if (gtk_clist_row_is_visible(ced->clist, ced->row) != GTK_VISIBILITY_FULL)
 		{
-		gtk_clist_moveto(ced->clist, row, -1, 0.0, 0.0);
+		nrow = ced->row;
+		if (ced->row != 0 && gtk_clist_row_is_visible(ced->clist, ced->row - 1) != GTK_VISIBILITY_NONE)
+			{
+			frow = 1.0;
+			}
 		}
-	else
+
+	clist_edit_get_geom(ced, &x, &y, &w, &h);
+	if (x < 0)
 		{
-		gtk_clist_moveto(ced->clist, row, -1, 1.0, 0.0);
+		ncol = ced->column;
 		}
+	else if (x + w > ced->clist->clist_window_width)
+		{
+		ncol = ced->column;
+		fcol = 1.0;
+		}
+
+	if (ncol > 0 || fcol > 0) gtk_clist_moveto(ced->clist, nrow, ncol, frow, fcol);
 }
 
 static void clist_edit_close(ClistEditData *ced)
@@ -138,7 +164,7 @@ gint clist_edit_by_row(GtkCList *clist, gint row, gint column,
 	ClistEditData *ced;
 	gint x, y, w, h;	/* geometry of cell within clist */
 	gint wx, wy;		/* geometry of clist from root window */
-	gchar *text;
+	gchar *text = NULL;
 	guint8 spacing = 0;
 	gint pix_width = 0;
 	GdkPixmap *pixmap = NULL;
@@ -149,10 +175,9 @@ gint clist_edit_by_row(GtkCList *clist, gint row, gint column,
 	if (!edit_func) return FALSE;
 	if (!GTK_WIDGET_VISIBLE(clist)) return FALSE;
 
-	if (!gtk_clist_get_text(clist, row, column, &text) &&
-	    !gtk_clist_get_pixtext(clist, row, column, &text, &spacing, &pixmap, &mask))
+	if (!gtk_clist_get_text(clist, row, column, &text))
 		{
-		return FALSE;
+		gtk_clist_get_pixtext(clist, row, column, &text, &spacing, &pixmap, &mask);
 		}
 
 	if (!text) text = "";
@@ -171,13 +196,13 @@ gint clist_edit_by_row(GtkCList *clist, gint row, gint column,
 	ced->row = row;
 	ced->column = column;
 
-	clist_edit_make_row_visible(ced, row);
+	clist_edit_make_row_visible(ced);
 
 	/* figure the position of the rename window
 	   (some borrowed from clist internals, may break in future ?) */
 
-	x = GTK_CLIST(ced->clist)->column[(ced->column)].area.x + GTK_CLIST(ced->clist)->hoffset + pix_width +spacing - COLUMN_SPACING - COLUMN_PADDING;
-	y = GTK_CLIST(ced->clist)->row_height * (row) + (((row) + 1) * 1) + GTK_CLIST(ced->clist)->voffset;
+	x = GTK_CLIST(ced->clist)->column[(ced->column)].area.x + GTK_CLIST(ced->clist)->hoffset + pix_width + spacing - COLUMN_SPACING - COLUMN_PADDING;
+	y = GTK_CLIST(ced->clist)->row_height * (ced->row) + (((ced->row) + 1) * 1) + GTK_CLIST(ced->clist)->voffset;
 
 	w = GTK_CLIST(ced->clist)->column[(ced->column)].area.width - pix_width - spacing + (COLUMN_SPACING + COLUMN_PADDING) * 2;
 	h = GTK_CLIST(ced->clist)->row_height;
@@ -241,3 +266,77 @@ void clist_edit_set_highlight(GtkWidget *clist, gint row, gint set)
 		}
 }
 
+static void shift_color(GdkColor *src)
+{
+	gshort cs = 0xffff / 100 * 10;	/* 10% shift */
+
+	/* up or down ? */
+	if (((gint)src->red + (gint)src->green + (gint)src->blue) / 3 > 0xffff / 2)
+		{
+		src->red = MAX(0 , src->red - cs);
+		src->green = MAX(0 , src->green - cs);
+		src->blue = MAX(0 , src->blue - cs);
+		}
+	else
+		{
+		src->red = MIN(0xffff, src->red + cs);
+		src->green = MIN(0xffff, src->green + cs);
+		src->blue = MIN(0xffff, src->blue + cs);
+		}
+}
+
+/* darkens or lightens color, so that rows can be grouped in the list
+ * esp. useful for alternating dark/light in lists
+ */
+void clist_edit_shift_color(GtkStyle *style)
+{
+	if (!style) return;
+
+	shift_color(&style->base[GTK_STATE_NORMAL]);
+	shift_color(&style->bg[GTK_STATE_SELECTED]);
+}
+
+GList *uig_list_insert_link(GList *list, GList *link, gpointer data)
+{
+	GList *new_list;
+
+	if (!list || link == list) return g_list_prepend(list, data);
+	if (!link) return g_list_append(list, data);
+
+	new_list = g_list_alloc ();
+	new_list->data = data;
+
+	if (link->prev)
+		{
+		link->prev->next = new_list;
+		new_list->prev = link->prev;
+		}
+	else
+		{
+		list = new_list;
+		}
+	link->prev = new_list;
+	new_list->next = link;
+
+	return list;
+}
+
+GList *uig_list_insert_list(GList *parent, GList *insert_link, GList *list)
+{
+	GList *end;
+
+	if (!insert_link) return g_list_concat(parent, list);
+	if (insert_link == parent) return g_list_concat(list, parent);
+	if (!parent) return list;
+	if (!list) return parent;
+
+	end  = g_list_last(list);
+
+	if (insert_link->prev) insert_link->prev->next = list;
+	list->prev = insert_link->prev;
+	insert_link->prev = end;
+	end->next = insert_link;
+
+	return parent;
+}
+
