@@ -1,6 +1,6 @@
 /*
  * (SLIK) SimpLIstic sKin functions
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -648,31 +648,6 @@ static void dest_filter_list_sync(Dest_Data *dd)
 	g_free(old_text);
 }
 
-static GList *uig_list_insert_link(GList *list, GList *link, gpointer data)
-{
-	GList *new_list;
-
-	if (!list || link == list) return g_list_prepend(list, data);
-	if (!link) return g_list_append(list, data);
-
-	new_list = g_list_alloc ();
-	new_list->data = data;
-
-	if (link->prev)
-		{
-		link->prev->next = new_list;
-		new_list->prev = link->prev;
-		}
-	else
-		{
-		list = new_list;
-		}
-	link->prev = new_list;
-	new_list->next = link;
-
-	return list;
-}
-
 static void dest_filter_add(Dest_Data *dd, const gchar *filter, const gchar *description, gint set)
 {
 	GList *work;
@@ -942,3 +917,4 @@ void path_selection_clear_filter(GtkWidget *entry)
 
 
 
+
