@@ -1,6 +1,6 @@
 /*
  * GQmpeg
- * (C) 2001 John Ellis
+ * (C) 2002 John Ellis
  *
  * Author: John Ellis
  *
@@ -72,10 +72,12 @@ static void key_pressed(GtkWidget *widget, GdkEventKey *event)
 				btn_iconify_pressed();
 				break;
 			case '-':
+			case GDK_KP_Subtract:
 				btn_balance_left_pressed();
 				btn_balance_released();
 				break;
 			case '+': case '=':
+			case GDK_KP_Add:
 				btn_balance_right_pressed();
 				btn_balance_released();
 				break;
@@ -117,10 +119,12 @@ static void key_pressed(GtkWidget *widget, GdkEventKey *event)
 				current_song_info_pressed();
 				break;
 			case '-':
+			case GDK_KP_Subtract:
 				btn_volume_down_pressed();
 				btn_volume_released();
 				break;
 			case '+': case '=':
+			case GDK_KP_Add:
 				btn_volume_up_pressed();
 				btn_volume_released();
 				break;
@@ -241,6 +245,7 @@ UIData *main_window_create(const gchar *skin_path)
 	UIData *ui;
 
 	ui = ui_new("GQmpeg", "gqmpeg", wm_decorations, "GQmpeg");
+	if (dock_mode) ui_moveable_set(ui, FALSE);
 	ui_set_icon(ui, (const char **)gqmpeg_icon_xpm, NULL);
 
 	gtk_signal_connect (GTK_OBJECT (ui->window), "delete_event", (GtkSignalFunc) destroy, ui);
